/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2007, 2008 Oracle.  All rights reserved.
 *
 * $Id: QueryExpressionTest.java,v 1.6 2008/04/30 13:02:58 lauren Exp $
 */

package dbxmltest;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlDocumentConfig;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlManager;
import com.sleepycat.dbxml.XmlQueryContext;
import com.sleepycat.dbxml.XmlQueryExpression;
import com.sleepycat.dbxml.XmlResults;
import com.sleepycat.dbxml.XmlTransaction;
import com.sleepycat.dbxml.XmlUpdateContext;
import com.sleepycat.dbxml.XmlValue;

public class QueryExpressionTest {
    private TestConfig hp;
    private XmlManager mgr = null;
    private XmlContainer cont = null;
    private static final String CON_NAME = "testData.dbxml";
    private static String docString = "<?xml version=\"1.0\" "
	+ "encoding=\"UTF-8\"?><a_node atr1=\"test\" atr2=\"test2\""
	+ "><b_node/><c_node>Other text</c_node><d_node/>"
	+ "</a_node>";
    private static String docName = "testDoc.xml";
    private static String query = "collection('" + CON_NAME + "')/a_node/c_node";

    @BeforeClass
    public static void setupClass() {
	TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
	System.out.println("Begin test XmlQueryExpression!");
    }

    @Before
    public void setUp() throws Throwable {
	hp = new TestConfig(XmlTestRunner.getEnvironmentType(),
		XmlTestRunner.isNodeContainer(), XmlTestRunner.getEnvironmentPath());
	mgr = hp.createManager();
	cont = hp.createContainer(CON_NAME, mgr);
    }

    @After
    public void tearDown() throws Throwable {
	hp.closeContainer(cont);
	hp.closeManager(mgr);
	hp.closeEnvironment();
	TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
    }

    @AfterClass
    public static void tearDownClass() {
	System.out.println("Finished test XmlQueryExpression!");
    }

    /*
     * Test for execute(XmlQueryContext queryContext) 
     * 
     */
    @Test
    public void testExecute() throws Throwable {
	XmlTransaction txn = null;
	XmlDocument doc = mgr.createDocument();
	doc.setName(docName);
	doc.setContent(docString);
	XmlUpdateContext xuc = mgr.createUpdateContext();
	XmlQueryContext context = mgr.createQueryContext();
	XmlQueryExpression qe = null;
	XmlResults re = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc, xuc);
		qe = mgr.prepare(txn, query, context);
		re = qe.execute(txn, context);
	    }else{
		cont.putDocument(doc, xuc);
		qe = mgr.prepare(query, context);
		re = qe.execute(context);
	    }
	    assertEquals(re.size(), 1);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(qe != null) qe.delete();
	    if(re != null) re.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for execute(XmlQueryContext queryContext, XmlDocumentConfig config)  
     * 
     */
    @Test
    public void testExecute_Dc() throws Throwable {
	XmlTransaction txn = null;
	XmlDocument doc = mgr.createDocument();
	doc.setName(docName);
	doc.setContent(docString);
	XmlUpdateContext xuc = mgr.createUpdateContext();
	XmlQueryContext context = mgr.createQueryContext();
	XmlQueryExpression qe = null;
	XmlResults re = null;
	XmlDocumentConfig xdc = new XmlDocumentConfig();

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc, xuc);
		qe = mgr.prepare(txn, query, context);
		re = qe.execute(txn, context, xdc);
	    }else{
		cont.putDocument(doc, xuc);
		qe = mgr.prepare(query, context);
		re = qe.execute(context, xdc);
	    }
	    assertEquals(re.size(), 1);
	} catch (XmlException e) {
	    if(qe != null) qe.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(re != null) re.delete();
	}

	//not valid XmlDocumentConfig flag
	xdc.setDocumentsOnly(true);
	try {
	    if(hp.isTransactional())
		re = qe.execute(txn, context, xdc);
	    else
		re = qe.execute(context, xdc);
	    fail("execute succeeded with an invalid document configuration.");
	} catch (XmlException e) {
	    assertNotNull(e.getMessage());
	    assertEquals(e.getErrorCode(), XmlException.INVALID_VALUE);
	}finally{
	    qe.delete();
	    if(re != null) re.delete();
	    if(txn != null) txn.commit();
	}
    }

    /*
     * Test for execute(XmlValue contextItem, XmlQueryContext queryContext)  
     * 
     */
    @Test
    public void testExecute_ValQc() throws Throwable {
	XmlTransaction txn = null;
	XmlDocument doc = mgr.createDocument();
	doc.setName(docName);
	doc.setContent(docString);
	XmlUpdateContext xuc = mgr.createUpdateContext();
	XmlQueryContext context = mgr.createQueryContext();
	XmlQueryExpression qe = null;
	XmlResults re = null;
	XmlValue val = new XmlValue();

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc, xuc);
		qe = mgr.prepare(txn, query, context);
		re = qe.execute(txn, val, context);
	    }else{
		cont.putDocument(doc, xuc);
		qe = mgr.prepare(query, context);
		re = qe.execute(val, context);
	    }
	    assertEquals(re.size(), 1);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(qe != null)qe.delete();
	    if(re != null) re.delete();
	    if(txn != null) txn.commit();
	}
    }

    /*
     * Test for execute(XmlValue contextItem, XmlQueryContext queryContext, XmlDocumentConfig config) 
     * 
     */
    @Test
    public void testExecute_ValQcDc() throws Throwable {
	XmlTransaction txn = null;
	XmlDocument doc = mgr.createDocument();
	doc.setName(docName);
	doc.setContent(docString);
	XmlUpdateContext xuc = mgr.createUpdateContext();
	XmlQueryContext context = mgr.createQueryContext();
	XmlQueryExpression qe = null;
	XmlResults re = null;
	XmlDocumentConfig xdc = new XmlDocumentConfig();
	XmlValue val = new XmlValue();

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc, xuc);
		qe = mgr.prepare(txn, query, context);
		re = qe.execute(txn, val, context , xdc);
	    }else{
		cont.putDocument(doc, xuc);
		qe = mgr.prepare(query, context);
		re = qe.execute(val, context , xdc);
	    }
	    assertEquals(re.size(), 1);
	} catch (XmlException e) {
	    if(qe != null) qe.delete();
	    if(txn != null) txn.commit();
	    throw e;
	} finally {
	    re.delete();
	    re = null;
	}
	
	//not valid XmlDocumentConfig flag
	xdc.setDocumentsOnly(true);
	try {
	    if(hp.isTransactional())
		re = qe.execute(txn, val, context, xdc);
	    else
		re = qe.execute(val, context, xdc);
	    fail("Not valid DocumentConfig");
	} catch (XmlException e) {
	    assertNotNull(e.getMessage());
	    assertEquals(e.getErrorCode(), XmlException.INVALID_VALUE);
	}finally{
	    if (qe != null) qe.delete();
	    if(re != null) re.delete();
	    if(txn != null) txn.commit();
	}
    }

    /*
     * Test for getQuery()  
     * 
     */
    @Test
    public void testGetQuery() throws Throwable {
	XmlTransaction txn = null;
	XmlQueryContext context = mgr.createQueryContext();
	XmlQueryExpression qe = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		qe = mgr.prepare(txn, query, context);
	    }else {
		qe = mgr.prepare(query, context);
	    }
	    assertEquals(qe.getQuery(), query);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(qe != null) qe.delete();
	    if(txn != null) txn.commit();
	}
    }

    /*
     * Test for getQueryPlan()  
     * 
     */
    @Test
    public void testGetQueryPlan() throws Throwable {
	XmlTransaction txn = null;
	XmlDocument doc = mgr.createDocument();
	doc.setName(docName);
	doc.setContent(docString);
	XmlUpdateContext xuc = mgr.createUpdateContext();
	XmlQueryContext context = mgr.createQueryContext();
	XmlQueryExpression qe = null;
	String plan = "<XQuery><QueryPlanToAST><StepQP axis=\"child\" "
	    + "name=\"c_node\" nodeType=\"element\"><StepQP"
	    + " axis=\"child\" name=\"a_node\" nodeType=\"elem"
	    + "ent\"><SequentialScanQP container=\"testData.db"
	    + "xml\" nodeType=\"document\"/></StepQP></Step"
	    + "QP></QueryPlanToAST></XQuery>";

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, doc, xuc);
		qe = mgr.prepare(txn, query, context);
	    }else{
		cont.putDocument(doc, xuc);
		qe = mgr.prepare(query, context);
	    }
	    assertNotNull(qe.getQueryPlan());
	    String queryPlan = qe.getQueryPlan();
	    //remove the '\n' for comparison
	    queryPlan = queryPlan.replaceAll(">\\s*<", "><");
	    queryPlan = queryPlan.replaceAll("\\n", "");
	    assertEquals(queryPlan, plan);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    qe.delete();
	    if(txn != null) txn.commit();
	}
    }

}
